unit scrollinglabel;

{$mode objfpc}{$H+}

interface

uses
  Classes, types, StdCtrls, ExtCtrls, Controls, LResources, LCLIntf;

type

  { TScrollingLabel }

  TScrollingLabel=class(TCustomLabel)
  private
    FOffset: integer;
    FScrollRefresh: integer;
    FScrolling: boolean;
    FStep: integer;
    FTimer: TTimer;
    FNeededWidth:integer;
    procedure OnTimer(Sender: TObject);
    procedure SetScrolling(AValue: boolean);
    procedure SetScrollRefresh(aValue: integer);
  protected
    procedure Paint; override;
    procedure AdjustSize; override;
    procedure DoMeasureTextPosition(var TextTop: integer; var TextLeft: integer);
      override;
  public
    constructor Create(anOwner: TComponent);override;
  published
    property ScrollRate: integer read FScrollRefresh write SetScrollRefresh default 500;
    property ScrollStep: integer read FStep write FStep default 1;
    property Scrolling: boolean read FScrolling write SetScrolling default True;
    property Align;
    property Alignment;
    property Anchors;
    //property AutoSize;
    property BidiMode;
    property BorderSpacing;
    property Caption;
    property Color;
    property Constraints;
    property DragCursor;
    property DragKind;
    property DragMode;
    property Enabled;
    property FocusControl;
    property Font;
    property Layout;
    property ParentBidiMode;
    property ParentColor;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowAccelChar;
    property ShowHint;
    property Transparent;
    property Visible;
    //property WordWrap;
    property OnChangeBounds;
    property OnClick;
    property OnContextPopup;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseEnter;
    property OnMouseLeave;
    property OnMouseMove;
    property OnMouseUp;
    property OnMouseWheel;
    property OnMouseWheelDown;
    property OnMouseWheelUp;
    property OnResize;
    property OnStartDrag;
    //property OptimalFill;
  end;

procedure register;

implementation

uses Graphics;

procedure register;
begin
  RegisterComponents('Additional', [TScrollingLabel]);
end;

procedure TScrollingLabel.OnTimer(Sender: TObject);
begin
  Dec(FOffset, FStep);
  if FOffset+FNeededwidth<0 then
    FOffset:=-FStep;
  Invalidate;
end;

procedure TScrollingLabel.SetScrolling(AValue: boolean);
begin
  if FScrolling=AValue then Exit;
  FScrolling:=AValue;
  FTimer.Enabled:=FScrolling and (FNeededWidth>Width);
  if not FScrolling then
    FOffset:=0;
  Invalidate;
end;

procedure TScrollingLabel.SetScrollRefresh(aValue: integer);
begin
  if FScrollRefresh=aValue then
    Exit;
  FScrollRefresh:=aValue;
  FTimer.Interval:=FScrollRefresh;
  Invalidate;
end;

procedure TScrollingLabel.Paint;
var
  txtStyle : TTextStyle;
  R : TRect;
  TextLeft, TextTop: integer;
  LabelText: string;
  OldFontColor: TColor;
  scroll: Boolean;
  oldtick: types.DWORD;
begin
  R := Rect(0,0,Width,Height);
  with Canvas do
  begin
    Brush.Color := Self.Color;
    if (Color<>clNone) and not Transparent then
    begin
      Brush.Style:=bsSolid;
      FillRect(R);
    end;
    Brush.Style:=bsClear;
    Font := Self.Font;
    FillChar(txtStyle,SizeOf(txtStyle),0);
    scroll:=FScrolling and (FNeededWidth>Self.Width);
    with txtStyle do
    begin
      if scroll then
        Alignment:=BidiFlipAlignment(taLeftJustify, UseRightToLeftAlignment)
      else
        Alignment := BidiFlipAlignment(Self.Alignment, UseRightToLeftAlignment);
      Layout := Self.Layout;
      Opaque := (Color<>clNone) and not Transparent;
      WordBreak := wordWrap;
      SingleLine:= not WordWrap and not HasMultiLine;
      Clipping := True;
      ShowPrefix := ShowAccelChar;
      SystemFont := False;
      RightToLeft := UseRightToLeftReading;
      ExpandTabs := True;
    end;
    DoMeasureTextPosition(TextTop, TextLeft);
    LabelText := GetLabelText;
    if scroll then
      LabelText:=LabelText+' '+LabelText;
    if not IsEnabled then
    begin
      writeln('*');
      OldFontColor := Font.Color;
      Font.Color := clBtnHighlight;
      TextRect(R, TextLeft + 1 + FOffset, TextTop + 1, LabelText, txtStyle);
      Font.Color := clBtnShadow;
      Font.Color := OldFontColor;
    end;
    oldtick:=GetTickCount;
    TextRect(R, TextLeft, TextTop, LabelText, txtStyle);
    Writeln(GetTickCount-oldtick);
  end;
end;

procedure TScrollingLabel.AdjustSize;
var dummy:integer;
begin
  inherited AdjustSize;
  if Parent=nil then
    exit;
  CalculateSize(0,FNeededWidth,dummy);
  if FNeededWidth<=Width then
  begin
    FTimer.Enabled:=false;
    FOffset:=0;
  end else
  begin
    FTimer.Enabled:=FScrolling;
    FNeededWidth:=FNeededWidth+Canvas.TextExtent(' ').Cx;
  end;
end;

procedure TScrollingLabel.DoMeasureTextPosition(var TextTop: integer;
  var TextLeft: integer);
begin
  inherited DoMeasureTextPosition(TextTop, TextLeft);
  Inc(TextLeft, FOffset);
end;

constructor TScrollingLabel.Create(anOwner: TComponent);
begin
  inherited Create(anOwner);
  AutoSize:=False;
  FScrollRefresh:=500;
  FScrolling:=true;
  FStep:=1;
  FTimer:=TTimer.Create(Self);
  FTimer.OnTimer:=@OnTimer;
  FTimer.Interval:=FScrollRefresh;
end;

//initialization
//{$i scrollinglabel.lrs}

end.
